@@ -26,11 +26,13 @@
 #include <math.h>
 
 #include <gst/audio/audio.h>
+#include "gstdvelements.h"
 #include "gstdvdemux.h"
 #include "gstsmptetimecode.h"
 
 /**
  * SECTION:element-dvdemux
+ * @title: dvdemux
  *
  * dvdemux splits raw DV into its audio and video components. The audio will be
  * decoded raw samples and the video will be encoded DV video.
@@ -38,12 +40,11 @@
  * This element can operate in both push and pull mode depending on the
  * capabilities of the upstream peer.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 filesrc location=test.dv ! dvdemux name=demux ! queue ! audioconvert ! alsasink demux. ! queue ! dvdec ! xvimagesink
  * ]| This pipeline decodes and renders the raw DV stream to an audio and a videosink.
- * </refsect2>
+ *
  */
 
 /* DV output has two modes, normal and wide. The resolution is the same in both
@@ -77,7 +78,7 @@
  *
  * For its part, GStreamer is concerned with accuracy and preservation of
  * information. This element outputs the 720x576 or 720x480 video that it
- * recieves, noting the proper aspect ratio. This should not be a problem for
+ * receives, noting the proper aspect ratio. This should not be a problem for
  * windowed applications, which can change size to fit the video. Applications
  * with fixed size requirements should decide whether to crop or pad which
  * an element such as videobox can do.
@@ -130,6 +131,8 @@ static GstStaticPadTemplate audio_src_temp = GST_STATIC_PAD_TEMPLATE ("audio",
 
 #define gst_dvdemux_parent_class parent_class
 G_DEFINE_TYPE (GstDVDemux, gst_dvdemux, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (dvdemux, "dvdemux", GST_RANK_PRIMARY,
+    GST_TYPE_DVDEMUX, dv_element_init (plugin));
 
 static void gst_dvdemux_finalize (GObject * object);
 
@@ -272,6 +275,7 @@ gst_dvdemux_reset (GstDVDemux * dvdemux)
   dvdemux->upstream_time_segment = FALSE;
   dvdemux->have_group_id = FALSE;
   dvdemux->group_id = G_MAXUINT;
+  dvdemux->tag_event = NULL;
 }
 
 static gboolean
@@ -294,16 +298,35 @@ have_group_id (GstDVDemux * demux)
   return demux->have_group_id;
 }
 
+static GstEvent *
+gst_dvdemux_create_global_tag_event (GstDVDemux * dvdemux)
+{
+  gchar rec_datetime[40];
+  GstDateTime *rec_dt;
+  GstTagList *tags;
+
+  tags = gst_tag_list_new (GST_TAG_CONTAINER_FORMAT, "DV", NULL);
+  gst_tag_list_set_scope (tags, GST_TAG_SCOPE_GLOBAL);
+
+  if (dv_get_recording_datetime (dvdemux->decoder, rec_datetime)) {
+    rec_dt = gst_date_time_new_from_iso8601_string (rec_datetime);
+    if (rec_dt) {
+      gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_DATE_TIME,
+          rec_dt, NULL);
+      gst_date_time_unref (rec_dt);
+    }
+  }
+
+  return gst_event_new_tag (tags);
+}
+
 static GstPad *
 gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template,
     GstCaps * caps)
 {
-  gboolean no_more_pads;
   GstPad *pad;
   GstEvent *event;
   gchar *stream_id;
-  gchar rec_datetime[40];
-  GstDateTime *rec_dt;
 
   pad = gst_pad_new_from_static_template (template, template->name_template);
 
@@ -330,35 +353,12 @@ gst_dvdemux_add_pad (GstDVDemux * dvdemux, GstStaticPadTemplate * template,
 
   gst_element_add_pad (GST_ELEMENT (dvdemux), pad);
 
-  no_more_pads =
-      (dvdemux->videosrcpad != NULL && template == &audio_src_temp) ||
-      (dvdemux->audiosrcpad != NULL && template == &video_src_temp);
+  if (!dvdemux->tag_event) {
+    dvdemux->tag_event = gst_dvdemux_create_global_tag_event (dvdemux);
+  }
 
-  if (no_more_pads)
-    gst_element_no_more_pads (GST_ELEMENT (dvdemux));
-
-  if (no_more_pads) {
-    GstTagList *tags;
-
-    tags = gst_tag_list_new (GST_TAG_CONTAINER_FORMAT, "DV", NULL);
-    gst_tag_list_set_scope (tags, GST_TAG_SCOPE_GLOBAL);
-
-    if (dv_get_recording_datetime (dvdemux->decoder, rec_datetime)) {
-      rec_dt = gst_date_time_new_from_iso8601_string (rec_datetime);
-      if (rec_dt) {
-        gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE, GST_TAG_DATE_TIME,
-            rec_dt, NULL);
-        gst_date_time_unref (rec_dt);
-      }
-    }
-
-    if (dvdemux->videosrcpad)
-      gst_pad_push_event (dvdemux->videosrcpad,
-          gst_event_new_tag (gst_tag_list_ref (tags)));
-    if (dvdemux->audiosrcpad)
-      gst_pad_push_event (dvdemux->audiosrcpad,
-          gst_event_new_tag (gst_tag_list_ref (tags)));
-    gst_tag_list_unref (tags);
+  if (dvdemux->tag_event) {
+    gst_pad_push_event (pad, gst_event_ref (dvdemux->tag_event));
   }
 
   return pad;
@@ -1124,7 +1124,7 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     gst_dvdemux_push_event (demux, new_event);
   }
 
-  /* if successfull seek, we update our real segment and push
+  /* if successful seek, we update our real segment and push
    * out the new segment. */
   if (res) {
     memcpy (&demux->time_segment, &seeksegment, sizeof (GstSegment));
@@ -1152,7 +1152,7 @@ gst_dvdemux_handle_pull_seek (GstDVDemux * demux, GstPad * pad,
     demux->need_segment = FALSE;
   }
 
-  /* and restart the task in case it got paused explicitely or by
+  /* and restart the task in case it got paused explicitly or by
    * the FLUSH_START event we pushed out. */
   gst_pad_start_task (demux->sinkpad, (GstTaskFunction) gst_dvdemux_loop,
       demux->sinkpad, NULL);
@@ -1279,6 +1279,10 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
       if (G_UNLIKELY (dvdemux->audiosrcpad == NULL)) {
         dvdemux->audiosrcpad =
             gst_dvdemux_add_pad (dvdemux, &audio_src_temp, caps);
+
+        if (dvdemux->videosrcpad && dvdemux->audiosrcpad)
+          gst_element_no_more_pads (GST_ELEMENT (dvdemux));
+
       } else {
         gst_pad_set_caps (dvdemux->audiosrcpad, caps);
       }
@@ -1309,7 +1313,6 @@ gst_dvdemux_demux_audio (GstDVDemux * dvdemux, GstBuffer * buffer,
 
     if (dvdemux->new_media || dvdemux->discont)
       GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
-
     ret = gst_pad_push (dvdemux->audiosrcpad, outbuf);
   } else {
     /* no samples */
@@ -1372,6 +1375,10 @@ gst_dvdemux_demux_video (GstDVDemux * dvdemux, GstBuffer * buffer,
     if (G_UNLIKELY (dvdemux->videosrcpad == NULL)) {
       dvdemux->videosrcpad =
           gst_dvdemux_add_pad (dvdemux, &video_src_temp, caps);
+
+      if (dvdemux->videosrcpad && dvdemux->audiosrcpad)
+        gst_element_no_more_pads (GST_ELEMENT (dvdemux));
+
     } else {
       gst_pad_set_caps (dvdemux->videosrcpad, caps);
     }
@@ -1533,10 +1540,18 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
       GST_SMPTE_TIME_CODE_SYSTEM_25 : GST_SMPTE_TIME_CODE_SYSTEM_30,
       &frame_number, &timecode);
 
-  next_ts = gst_util_uint64_scale_int (
-      (dvdemux->frame_offset + 1) * GST_SECOND,
-      dvdemux->framerate_denominator, dvdemux->framerate_numerator);
-  duration = next_ts - dvdemux->time_segment.position;
+  if (dvdemux->time_segment.rate < 0) {
+    next_ts = gst_util_uint64_scale_int (
+        (dvdemux->frame_offset >
+            0 ? dvdemux->frame_offset - 1 : 0) * GST_SECOND,
+        dvdemux->framerate_denominator, dvdemux->framerate_numerator);
+    duration = dvdemux->time_segment.position - next_ts;
+  } else {
+    next_ts = gst_util_uint64_scale_int (
+        (dvdemux->frame_offset + 1) * GST_SECOND,
+        dvdemux->framerate_denominator, dvdemux->framerate_numerator);
+    duration = next_ts - dvdemux->time_segment.position;
+  }
 
   gst_buffer_map (buffer, &map, GST_MAP_READ);
   dv_parse_packs (dvdemux->decoder, map.data);
@@ -1569,10 +1584,22 @@ gst_dvdemux_demux_frame (GstDVDemux * dvdemux, GstBuffer * buffer)
 
   dvdemux->discont = FALSE;
   dvdemux->time_segment.position = next_ts;
-  dvdemux->frame_offset++;
+
+  if (dvdemux->time_segment.rate < 0) {
+    if (dvdemux->frame_offset > 0)
+      dvdemux->frame_offset--;
+    else
+      GST_WARNING_OBJECT (dvdemux,
+          "Got before frame offset 0 in reverse playback");
+  } else {
+    dvdemux->frame_offset++;
+  }
 
   /* check for the end of the segment */
-  if (dvdemux->time_segment.stop != -1 && next_ts > dvdemux->time_segment.stop)
+  if ((dvdemux->time_segment.rate > 0 && dvdemux->time_segment.stop != -1
+          && next_ts > dvdemux->time_segment.stop)
+      || (dvdemux->time_segment.rate < 0
+          && dvdemux->time_segment.start > next_ts))
     ret = GST_FLOW_EOS;
   else
     ret = GST_FLOW_OK;
@@ -1776,7 +1803,7 @@ gst_dvdemux_loop (GstPad * pad)
         if (!gst_dvdemux_handle_pull_seek (dvdemux, dvdemux->videosrcpad,
                 event)) {
           GST_ELEMENT_WARNING (dvdemux, STREAM, DECODE, (NULL),
-              ("Error perfoming initial seek"));
+              ("Error performing initial seek"));
         }
         gst_event_unref (event);
 
@@ -1998,6 +2025,12 @@ gst_dvdemux_change_state (GstElement * element, GstStateChange transition)
       dvdemux->decoder = NULL;
 
       gst_dvdemux_remove_pads (dvdemux);
+
+      if (dvdemux->tag_event) {
+        gst_event_unref (dvdemux->tag_event);
+        dvdemux->tag_event = NULL;
+      }
+
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
     {
