@@ -197,6 +197,7 @@ static void
 gst_vpx_dec_init (GstVPXDec * gst_vpx_dec)
 {
   GstVideoDecoder *decoder = (GstVideoDecoder *) gst_vpx_dec;
+  GstVPXDecClass *vpxclass = GST_VPX_DEC_GET_CLASS (gst_vpx_dec);
 
   GST_DEBUG_OBJECT (gst_vpx_dec, "gst_vpx_dec_init");
   gst_video_decoder_set_packetized (decoder, TRUE);
@@ -205,6 +206,11 @@ gst_vpx_dec_init (GstVPXDec * gst_vpx_dec)
   gst_vpx_dec->deblocking_level = DEFAULT_DEBLOCKING_LEVEL;
   gst_vpx_dec->noise_level = DEFAULT_NOISE_LEVEL;
 
+  if (vpxclass->get_needs_sync_point) {
+    gst_video_decoder_set_needs_sync_point (GST_VIDEO_DECODER (gst_vpx_dec),
+        vpxclass->get_needs_sync_point (gst_vpx_dec));
+  }
+
   gst_video_decoder_set_needs_format (decoder, TRUE);
   gst_video_decoder_set_use_default_pad_acceptcaps (decoder, TRUE);
   GST_PAD_SET_ACCEPT_TEMPLATE (GST_VIDEO_DECODER_SINK_PAD (decoder));
@@ -280,6 +286,7 @@ gst_vpx_dec_start (GstVideoDecoder * decoder)
 
   GST_DEBUG_OBJECT (gst_vpx_dec, "start");
   gst_vpx_dec->decoder_inited = FALSE;
+  gst_vpx_dec->safe_remap = FALSE;
 
   return TRUE;
 }
@@ -395,6 +402,15 @@ gst_vpx_dec_prepare_image (GstVPXDec * dec, const vpx_image_t * img)
 
   buffer = gst_buffer_ref (frame->buffer);
 
+  /* FIXME: an atomic remap would be preferable, for now we simply
+   * remap the buffer from RW to RO when using a sysmem allocator,
+   * in order to avoid a useless memcpy in GstVideoDecoder.
+   */
+  if (dec->safe_remap) {
+    gst_buffer_unmap (buffer, &frame->info);
+    gst_buffer_map (buffer, &frame->info, GST_MAP_READ);
+  }
+
   vmeta = gst_buffer_get_video_meta (buffer);
   vmeta->format = GST_VIDEO_INFO_FORMAT (info);
   vmeta->width = GST_VIDEO_INFO_WIDTH (info);
@@ -444,6 +460,9 @@ gst_vpx_dec_get_buffer_cb (gpointer priv, gsize min_size,
       allocator = NULL;
     }
 
+    dec->safe_remap = (allocator == NULL
+        || !g_strcmp0 (allocator->mem_type, GST_ALLOCATOR_SYSMEM));
+
     pool = gst_buffer_pool_new ();
     config = gst_buffer_pool_get_config (pool);
     gst_buffer_pool_config_set_allocator (config, allocator, &params);
@@ -581,14 +600,10 @@ gst_vpx_dec_open_codec (GstVPXDec * dec, GstVideoCodecFrame * frame)
   gst_buffer_unmap (frame->input_buffer, &minfo);
 
   if (status != VPX_CODEC_OK) {
-    GST_WARNING_OBJECT (dec, "VPX preprocessing error: %s",
-        gst_vpx_error_name (status));
-    return GST_FLOW_CUSTOM_SUCCESS_1;
-  }
-  if (!stream_info.is_kf) {
-    GST_WARNING_OBJECT (dec, "No keyframe, skipping");
+    GST_VPX_DEC_WARN (dec, "VPX preprocessing error", status);
     return GST_FLOW_CUSTOM_SUCCESS_1;
   }
+
   if (stream_info.w == 0 || stream_info.h == 0) {
     /* For VP8 it's possible to signal width or height to be 0, but it does
      * not make sense to do so. For VP9 it's impossible. Hence, we most likely
@@ -623,9 +638,12 @@ gst_vpx_dec_open_codec (GstVPXDec * dec, GstVideoCodecFrame * frame)
   status =
       vpx_codec_dec_init (&dec->decoder, vpxclass->codec_algo, &cfg, flags);
   if (status != VPX_CODEC_OK) {
-    GST_ELEMENT_ERROR (dec, LIBRARY, INIT,
-        ("Failed to initialize VP8 decoder"), ("%s",
-            gst_vpx_error_name (status)));
+    GST_ELEMENT_ERROR_WITH_DETAILS (dec, LIBRARY, INIT,
+        ("Failed to initialize VP8 decoder"), ("%s (details: %s)",
+            gst_vpx_error_name (status),
+            GST_STR_NULL (dec->decoder.err_detail)), ("error", G_TYPE_STRING,
+            gst_vpx_error_name (status), "details", G_TYPE_STRING,
+            GST_STR_NULL (dec->decoder.err_detail), NULL));
     return GST_FLOW_ERROR;
   }
 
@@ -638,8 +656,7 @@ gst_vpx_dec_open_codec (GstVPXDec * dec, GstVideoCodecFrame * frame)
 
     status = vpx_codec_control (&dec->decoder, VP8_SET_POSTPROC, &pp_cfg);
     if (status != VPX_CODEC_OK) {
-      GST_WARNING_OBJECT (dec, "Couldn't set postprocessing settings: %s",
-          gst_vpx_error_name (status));
+      GST_VPX_DEC_WARN (dec, "Couldn't set postprocessing settings", status);
     }
   }
   vpx_codec_set_frame_buffer_functions (&dec->decoder,
@@ -672,6 +689,12 @@ gst_vpx_dec_handle_frame (GstVideoDecoder * decoder, GstVideoCodecFrame * frame)
   if (!dec->decoder_inited) {
     ret = vpxclass->open_codec (dec, frame);
     if (ret == GST_FLOW_CUSTOM_SUCCESS_1) {
+      GstVideoDecoderRequestSyncPointFlags flags = 0;
+
+      if (gst_video_decoder_get_needs_sync_point (decoder))
+        flags |= GST_VIDEO_DECODER_REQUEST_SYNC_POINT_DISCARD_INPUT;
+
+      gst_video_decoder_request_sync_point (decoder, frame, flags);
       gst_video_decoder_drop_frame (decoder, frame);
       return GST_FLOW_OK;
     } else if (ret != GST_FLOW_OK) {
@@ -701,8 +724,17 @@ gst_vpx_dec_handle_frame (GstVideoDecoder * decoder, GstVideoCodecFrame * frame)
   gst_buffer_unmap (frame->input_buffer, &minfo);
 
   if (status) {
-    GST_VIDEO_DECODER_ERROR (decoder, 1, LIBRARY, ENCODE,
-        ("Failed to decode frame"), ("%s", gst_vpx_error_name (status)), ret);
+    GstVideoDecoderRequestSyncPointFlags flags = 0;
+
+    GST_VIDEO_DECODER_ERROR (decoder, 1, STREAM, DECODE,
+        ("Failed to decode frame"), ("%s (details: %s)",
+            gst_vpx_error_name (status),
+            GST_STR_NULL (dec->decoder.err_detail)), ret);
+
+    if (gst_video_decoder_get_needs_sync_point (decoder))
+      flags |= GST_VIDEO_DECODER_REQUEST_SYNC_POINT_DISCARD_INPUT;
+
+    gst_video_decoder_request_sync_point (decoder, frame, flags);
     gst_video_codec_frame_unref (frame);
     return ret;
   }
@@ -711,7 +743,7 @@ gst_vpx_dec_handle_frame (GstVideoDecoder * decoder, GstVideoCodecFrame * frame)
   if (img) {
     if (vpxclass->get_frame_format (dec, img, &fmt) == FALSE) {
       vpx_img_free (img);
-      GST_ELEMENT_ERROR (decoder, LIBRARY, ENCODE,
+      GST_ELEMENT_ERROR (decoder, STREAM, DECODE,
           ("Failed to decode frame"), ("Unsupported color format %d",
               img->fmt));
       gst_video_codec_frame_unref (frame);
