@@ -1,5 +1,5 @@
 /* GStreamer Adaptive Multi-Rate parser plugin
- * Copyright (C) 2006 Edgard Lima <edgard.lima@indt.org.br>
+ * Copyright (C) 2006 Edgard Lima <edgard.lima@gmail.com>
  * Copyright (C) 2008 Nokia Corporation. All rights reserved.
  *
  * Contact: Stefan Kost <stefan.kost@nokia.com>
@@ -22,18 +22,18 @@
 
 /**
  * SECTION:element-amrparse
+ * @title: amrparse
  * @short_description: AMR parser
  * @see_also: #GstAmrnbDec, #GstAmrnbEnc
  *
  * This is an AMR parser capable of handling both narrow-band and wideband
  * formats.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 filesrc location=abc.amr ! amrparse ! amrdec ! audioresample ! audioconvert ! alsasink
  * ]|
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,7 @@
 
 #include <string.h>
 
+#include "gstaudioparserselements.h"
 #include "gstamrparse.h"
 #include <gst/pbutils/pbutils.h>
 
@@ -85,6 +86,8 @@ static GstFlowReturn gst_amr_parse_pre_push_frame (GstBaseParse * parse,
     GstBaseParseFrame * frame);
 
 G_DEFINE_TYPE (GstAmrParse, gst_amr_parse, GST_TYPE_BASE_PARSE);
+GST_ELEMENT_REGISTER_DEFINE (amrparse, "amrparse",
+    GST_RANK_PRIMARY + 1, GST_TYPE_AMR_PARSE);
 
 /**
  * gst_amr_parse_class_init:
@@ -100,10 +103,8 @@ gst_amr_parse_class_init (GstAmrParseClass * klass)
   GST_DEBUG_CATEGORY_INIT (amrparse_debug, "amrparse", 0,
       "AMR-NB audio stream parser");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (element_class, &sink_template);
+  gst_element_class_add_static_pad_template (element_class, &src_template);
 
   gst_element_class_set_static_metadata (element_class,
       "AMR audio stream parser", "Codec/Parser/Audio",
@@ -426,10 +427,19 @@ gst_amr_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     GstTagList *taglist;
     GstCaps *caps;
 
-    taglist = gst_tag_list_new_empty ();
-
     /* codec tag */
     caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (parse));
+    if (G_UNLIKELY (caps == NULL)) {
+      if (GST_PAD_IS_FLUSHING (GST_BASE_PARSE_SRC_PAD (parse))) {
+        GST_INFO_OBJECT (parse, "Src pad is flushing");
+        return GST_FLOW_FLUSHING;
+      } else {
+        GST_INFO_OBJECT (parse, "Src pad is not negotiated!");
+        return GST_FLOW_NOT_NEGOTIATED;
+      }
+    }
+
+    taglist = gst_tag_list_new_empty ();
     gst_pb_utils_add_codec_description_to_tag_list (taglist,
         GST_TAG_AUDIO_CODEC, caps);
     gst_caps_unref (caps);
@@ -441,5 +451,7 @@ gst_amr_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
     amrparse->sent_codec_tag = TRUE;
   }
 
+  frame->flags |= GST_BASE_PARSE_FRAME_FLAG_CLIP;
+
   return GST_FLOW_OK;
 }
