@@ -27,6 +27,7 @@
 
 /**
  * SECTION:element-dtmfsrc
+ * @title: dtmfsrc
  * @see_also: rtpdtmsrc, rtpdtmfmuxx
  *
  * The DTMFSrc element generates DTMF (ITU-T Q.23 Specification) tone packets on request
@@ -36,70 +37,28 @@
  * structure of name "dtmf-event" with fields set according to the following
  * table:
  *
- * <informaltable>
- * <tgroup cols='4'>
- * <colspec colname='Name' />
- * <colspec colname='Type' />
- * <colspec colname='Possible values' />
- * <colspec colname='Purpose' />
- * <thead>
- * <row>
- * <entry>Name</entry>
- * <entry>GType</entry>
- * <entry>Possible values</entry>
- * <entry>Purpose</entry>
- * </row>
- * </thead>
- * <tbody>
- * <row>
- * <entry>type</entry>
- * <entry>G_TYPE_INT</entry>
- * <entry>0-1</entry>
- * <entry>The application uses this field to specify which of the two methods
- * specified in RFC 2833 to use. The value should be 0 for tones and 1 for
- * named events. Tones are specified by their frequencies and events are specied
- * by their number. This element can only take events as input. Do not confuse
- * with "method" which specified the output.
- * </entry>
- * </row>
- * <row>
- * <entry>number</entry>
- * <entry>G_TYPE_INT</entry>
- * <entry>0-15</entry>
- * <entry>The event number.</entry>
- * </row>
- * <row>
- * <entry>volume</entry>
- * <entry>G_TYPE_INT</entry>
- * <entry>0-36</entry>
- * <entry>This field describes the power level of the tone, expressed in dBm0
- * after dropping the sign. Power levels range from 0 to -63 dBm0. The range of
- * valid DTMF is from 0 to -36 dBm0. Can be omitted if start is set to FALSE.
- * </entry>
- * </row>
- * <row>
- * <entry>start</entry>
- * <entry>G_TYPE_BOOLEAN</entry>
- * <entry>True or False</entry>
- * <entry>Whether the event is starting or ending.</entry>
- * </row>
- * <row>
- * <entry>method</entry>
- * <entry>G_TYPE_INT</entry>
- * <entry>2</entry>
- * <entry>The method used for sending event, this element will react if this
- * field is absent or 2.
- * </entry>
- * </row>
- * </tbody>
- * </tgroup>
- * </informaltable>
+ * * `type` (G_TYPE_INT, 0-1): The application uses this field to specify which of the two methods
+ *   specified in RFC 2833 to use. The value should be 0 for tones and 1 for
+ *   named events. Tones are specified by their frequencies and events are specified
+ *   by their number. This element can only take events as input. Do not confuse
+ *   with "method" which specified the output.
+ *
+ * * `number` (G_TYPE_INT, 0-15): The event number.
+ *
+ * * `volume` (G_TYPE_INT, 0-36): This field describes the power level of the tone, expressed in dBm0
+ *   after dropping the sign. Power levels range from 0 to -63 dBm0. The range of
+ *   valid DTMF is from 0 to -36 dBm0. Can be omitted if start is set to FALSE.
+ *
+ * * `start` (G_TYPE_BOOLEAN, True or False): Whether the event is starting or ending.
+ *
+ * * `method` (G_TYPE_INT, 2): The method used for sending event, this element will react if this
+ *   field is absent or 2.
  *
  * For example, the following code informs the pipeline (and in turn, the
  * DTMFSrc element inside the pipeline) about the start of a DTMF named
  * event '1' of volume -25 dBm0:
  *
- * <programlisting>
+ * |[
  * structure = gst_structure_new ("dtmf-event",
  *                    "type", G_TYPE_INT, 1,
  *                    "number", G_TYPE_INT, 1,
@@ -108,7 +67,7 @@
  *
  * event = gst_event_new_custom (GST_EVENT_CUSTOM_UPSTREAM, structure);
  * gst_element_send_event (pipeline, event);
- * </programlisting>
+ * ]|
  *
  * When a DTMF tone actually starts or stop, a "dtmf-event-processed"
  * element #GstMessage with the same fields as the "dtmf-event"
@@ -208,11 +167,14 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw, "
         "format = (string) \"" GST_AUDIO_NE (S16) "\", "
-        "rate = " GST_AUDIO_RATE_RANGE ", " "channels = (int) 1")
+        "rate = " GST_AUDIO_RATE_RANGE ", " "channels = (int) 1, "
+        "layout = (string)interleaved")
     );
 
 #define parent_class gst_dtmf_src_parent_class
 G_DEFINE_TYPE (GstDTMFSrc, gst_dtmf_src, GST_TYPE_BASE_SRC);
+GST_ELEMENT_REGISTER_DEFINE (dtmfsrc, "dtmfsrc", GST_RANK_NONE,
+    GST_TYPE_DTMF_SRC);
 
 static void gst_dtmf_src_finalize (GObject * object);
 
@@ -234,6 +196,7 @@ static gboolean gst_dtmf_src_unlock (GstBaseSrc * src);
 
 static gboolean gst_dtmf_src_unlock_stop (GstBaseSrc * src);
 static gboolean gst_dtmf_src_negotiate (GstBaseSrc * basesrc);
+static gboolean gst_dtmf_src_query (GstBaseSrc * basesrc, GstQuery * query);
 
 
 static void
@@ -250,8 +213,8 @@ gst_dtmf_src_class_init (GstDTMFSrcClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (gst_dtmf_src_debug, "dtmfsrc", 0, "dtmfsrc element");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_dtmf_src_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_dtmf_src_template);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "DTMF tone generator", "Source/Audio", "Generates DTMF tones",
@@ -277,13 +240,14 @@ gst_dtmf_src_class_init (GstDTMFSrcClass * klass)
   gstbasesrc_class->event = GST_DEBUG_FUNCPTR (gst_dtmf_src_handle_event);
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_dtmf_src_create);
   gstbasesrc_class->negotiate = GST_DEBUG_FUNCPTR (gst_dtmf_src_negotiate);
+  gstbasesrc_class->query = GST_DEBUG_FUNCPTR (gst_dtmf_src_query);
 }
 
 static void
 event_free (GstDTMFSrcEvent * event)
 {
   if (event)
-    g_slice_free (GstDTMFSrcEvent, event);
+    g_free (event);
 }
 
 static void
@@ -517,7 +481,7 @@ gst_dtmf_src_add_start_event (GstDTMFSrc * dtmfsrc, gint event_number,
     gint event_volume)
 {
 
-  GstDTMFSrcEvent *event = g_slice_new0 (GstDTMFSrcEvent);
+  GstDTMFSrcEvent *event = g_new0 (GstDTMFSrcEvent, 1);
   event->event_type = DTMF_EVENT_TYPE_START;
   event->sample = 0;
   event->event_number = CLAMP (event_number, MIN_EVENT, MAX_EVENT);
@@ -530,7 +494,7 @@ static void
 gst_dtmf_src_add_stop_event (GstDTMFSrc * dtmfsrc)
 {
 
-  GstDTMFSrcEvent *event = g_slice_new0 (GstDTMFSrcEvent);
+  GstDTMFSrcEvent *event = g_new0 (GstDTMFSrcEvent, 1);
   event->event_type = DTMF_EVENT_TYPE_STOP;
   event->sample = 0;
   event->event_number = 0;
@@ -725,7 +689,7 @@ gst_dtmf_src_create (GstBaseSrc * basesrc, guint64 offset,
           break;
       }
       if (event)
-        g_slice_free (GstDTMFSrcEvent, event);
+        g_free (event);
     } else if (dtmfsrc->last_event->packet_count * dtmfsrc->interval >=
         MIN_DUTY_CYCLE) {
       event = g_async_queue_try_pop (dtmfsrc->event_queue);
@@ -739,7 +703,7 @@ gst_dtmf_src_create (GstBaseSrc * basesrc, guint64 offset,
             gst_dtmf_src_post_message (dtmfsrc, "dtmf-event-dropped", event);
             break;
           case DTMF_EVENT_TYPE_STOP:
-            g_slice_free (GstDTMFSrcEvent, dtmfsrc->last_event);
+            g_free (dtmfsrc->last_event);
             dtmfsrc->last_event = NULL;
             gst_dtmf_src_post_message (dtmfsrc, "dtmf-event-processed", event);
             break;
@@ -759,7 +723,7 @@ gst_dtmf_src_create (GstBaseSrc * basesrc, guint64 offset,
 
             break;
         }
-        g_slice_free (GstDTMFSrcEvent, event);
+        g_free (event);
       }
     }
   } while (dtmfsrc->last_event == NULL);
@@ -809,7 +773,7 @@ paused:
   if (dtmfsrc->last_event) {
     GST_DEBUG_OBJECT (dtmfsrc, "Stopping current event");
     /* Don't forget to release the stream lock */
-    g_slice_free (GstDTMFSrcEvent, dtmfsrc->last_event);
+    g_free (dtmfsrc->last_event);
     dtmfsrc->last_event = NULL;
   }
 
@@ -833,7 +797,7 @@ gst_dtmf_src_unlock (GstBaseSrc * src)
   GST_OBJECT_UNLOCK (dtmfsrc);
 
   GST_DEBUG_OBJECT (dtmfsrc, "Pushing the PAUSE_TASK event on unlock request");
-  event = g_slice_new0 (GstDTMFSrcEvent);
+  event = g_new0 (GstDTMFSrcEvent, 1);
   event->event_type = DTMF_EVENT_TYPE_PAUSE_TASK;
   g_async_queue_push (dtmfsrc->event_queue, event);
 
@@ -894,6 +858,33 @@ gst_dtmf_src_negotiate (GstBaseSrc * basesrc)
   return ret;
 }
 
+static gboolean
+gst_dtmf_src_query (GstBaseSrc * basesrc, GstQuery * query)
+{
+  GstDTMFSrc *dtmfsrc = GST_DTMF_SRC (basesrc);
+  gboolean res = FALSE;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_LATENCY:
+    {
+      GstClockTime latency;
+
+      latency = dtmfsrc->interval * GST_MSECOND;
+      gst_query_set_latency (query, gst_base_src_is_live (basesrc), latency,
+          GST_CLOCK_TIME_NONE);
+      GST_DEBUG_OBJECT (dtmfsrc, "Reporting latency of %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (latency));
+      res = TRUE;
+    }
+      break;
+    default:
+      res = GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
+      break;
+  }
+
+  return res;
+}
+
 static GstStateChangeReturn
 gst_dtmf_src_change_state (GstElement * element, GstStateChange transition)
 {
@@ -911,7 +902,7 @@ gst_dtmf_src_change_state (GstElement * element, GstStateChange transition)
 
       while (event != NULL) {
         gst_dtmf_src_post_message (dtmfsrc, "dtmf-event-dropped", event);
-        g_slice_free (GstDTMFSrcEvent, event);
+        g_free (event);
         event = g_async_queue_try_pop (dtmfsrc->event_queue);
       }
       dtmfsrc->last_event_was_start = FALSE;
@@ -938,7 +929,7 @@ gst_dtmf_src_change_state (GstElement * element, GstStateChange transition)
 
       while (event != NULL) {
         gst_dtmf_src_post_message (dtmfsrc, "dtmf-event-dropped", event);
-        g_slice_free (GstDTMFSrcEvent, event);
+        g_free (event);
         event = g_async_queue_try_pop (dtmfsrc->event_queue);
       }
       dtmfsrc->last_event_was_start = FALSE;
@@ -960,10 +951,3 @@ failure:
     return result;
   }
 }
-
-gboolean
-gst_dtmf_src_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "dtmfsrc",
-      GST_RANK_NONE, GST_TYPE_DTMF_SRC);
-}
