@@ -24,22 +24,22 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
  * SECTION:element-rippletv
+ * @title: rippletv
  *
  * RippleTV does ripple mark effect on the video input. The ripple is caused
  * by motion or random rain drops.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v videotestsrc ! rippletv ! videoconvert ! autovideosink
  * ]| This pipeline shows the effect of rippletv on a test stream.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -83,6 +83,8 @@ gst_rippletv_mode_get_type (void)
 
 #define gst_rippletv_parent_class parent_class
 G_DEFINE_TYPE (GstRippleTV, gst_rippletv, GST_TYPE_VIDEO_FILTER);
+GST_ELEMENT_REGISTER_DEFINE (rippletv, "rippletv", GST_RANK_NONE,
+    GST_TYPE_RIPPLETV);
 
 static GstStaticPadTemplate gst_rippletv_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -460,24 +462,20 @@ gst_rippletv_set_info (GstVideoFilter * vfilter, GstCaps * incaps,
 
   /* we over allocate the buffers, as the render code does not handle clipping
    * very well */
-  if (filter->map)
-    g_free (filter->map);
+  g_free (filter->map);
   filter->map = g_new0 (gint, (1 + filter->map_h) * filter->map_w * 3);
 
   filter->map1 = filter->map;
   filter->map2 = filter->map + filter->map_w * filter->map_h;
   filter->map3 = filter->map + filter->map_w * filter->map_h * 2;
 
-  if (filter->vtable)
-    g_free (filter->vtable);
+  g_free (filter->vtable);
   filter->vtable = g_new0 (gint8, (1 + filter->map_h) * filter->map_w * 2);
 
-  if (filter->background)
-    g_free (filter->background);
+  g_free (filter->background);
   filter->background = g_new0 (gint16, width * (height + 1));
 
-  if (filter->diff)
-    g_free (filter->diff);
+  g_free (filter->diff);
   filter->diff = g_new0 (guint8, width * (height + 1));
   GST_OBJECT_UNLOCK (filter);
 
@@ -507,20 +505,16 @@ gst_rippletv_finalize (GObject * object)
 {
   GstRippleTV *filter = GST_RIPPLETV (object);
 
-  if (filter->map)
-    g_free (filter->map);
+  g_free (filter->map);
   filter->map = NULL;
 
-  if (filter->vtable)
-    g_free (filter->vtable);
+  g_free (filter->vtable);
   filter->vtable = NULL;
 
-  if (filter->background)
-    g_free (filter->background);
+  g_free (filter->background);
   filter->background = NULL;
 
-  if (filter->diff)
-    g_free (filter->diff);
+  g_free (filter->diff);
   filter->diff = NULL;
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -594,10 +588,10 @@ gst_rippletv_class_init (GstRippleTVClass * klass)
       "FUKUCHI, Kentarou <fukuchi@users.sourceforge.net>, "
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rippletv_sink_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_rippletv_src_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rippletv_sink_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_rippletv_src_template);
 
   trans_class->start = GST_DEBUG_FUNCPTR (gst_rippletv_start);
 
@@ -606,6 +600,8 @@ gst_rippletv_class_init (GstRippleTVClass * klass)
       GST_DEBUG_FUNCPTR (gst_rippletv_transform_frame);
 
   setTable ();
+
+  gst_type_mark_as_plugin_api (GST_TYPE_RIPPLETV_MODE, 0);
 }
 
 static void
