@@ -560,80 +560,91 @@ typedef enum {
 
 typedef struct _GstMatroskaTrackContext GstMatroskaTrackContext;
 
-/* TODO: check if all fields are used */
 struct _GstMatroskaTrackContext {
-  GstPad       *pad;
-  GstCaps      *caps;
-  guint         index;
-  /* reverse playback */
-  GstClockTime  from_time;
-  gint64                   from_offset;
-  gint64                   to_offset;
-
-  GArray       *index_table;
-
-  gint          index_writer_id;
-
-  /* some often-used info */
-  gchar        *codec_id, *codec_name, *name, *language;
-  gpointer      codec_priv;
-  gsize         codec_priv_size;
-  gpointer      codec_state;
-  gsize         codec_state_size;
+  //
+  // common fields
+  //
+  gchar         *codec_id, *codec_name, *name, *language;
+  gpointer       codec_priv;
+  gsize          codec_priv_size;
   GstMatroskaTrackType type;
-  guint64       uid, num;
+  guint64        uid, num;
   GstMatroskaTrackFlags flags;
-  guint64       default_duration;
-  guint64       pos;
-  gdouble       timecodescale;
-  guint64       seek_preroll;
-  guint64       codec_delay;
+  guint64        default_duration;
+  gdouble        timecodescale;
+  guint64        seek_preroll;
+  guint64        codec_delay;
 
-  gboolean      set_discont; /* TRUE = set DISCONT flag on next buffer */
+  /* for compatibility with VFW files, where timestamp represents DTS */
+  gboolean       dts_only;
+
+  //
+  // Used only by demuxer / parser
+  //
+
+  GstPad        *pad;
+  GstCaps       *caps;
+  guint          index;
+  guint64        pos;
+
+  /* reverse playback */
+  GstClockTime   from_time;
+  gint64         from_offset;
+  gint64         to_offset;
+
+  GArray        *index_table;
+
+  gint           index_writer_id;
+
+  gpointer       codec_state;
+  gsize          codec_state_size;
+
+  gboolean       set_discont; /* TRUE = set DISCONT flag on next buffer */
 
   /* Queue to save the GST_PROTECTION events which will be sent before the first source buffer */
   GQueue         protection_event_queue;
   /* Protection information structure which will be added in protection metadata for each encrypted buffer */
-  GstStructure * protection_info;
+  GstStructure  *protection_info;
 
   /* Stream header buffer, to put into caps and send before any other buffers */
-  GstBufferList * stream_headers;
-  gboolean        send_stream_headers;
+  GstBufferList *stream_headers;
+  gboolean       send_stream_headers;
+
+  /* List of tags for this stream */
+  GstTagList    *tags;
+  /* Tags changed and should be pushed again */
+  gboolean       tags_changed;
+
+  /* A GArray of GstMatroskaTrackEncoding structures which contain the
+   * encoding (compression/encryption) settings for this track, if any */
+  GArray        *encodings;
+
+  /* Whether the stream is EOS */
+  gboolean       eos;
+
+  /* indicate that the track is raw (jpeg,raw variants) and so pts=dts */
+  gboolean       intra_only;
+
+  /* any alignment we need our output buffers to have */
+  gint           alignment;
 
   /* Special flag for VobSub, for which we have to send colour table info
    * (if available) first before sending any data, and just testing
    * for time == 0 is not enough to detect that. Used by demuxer */
-  gboolean      send_dvd_event;
-
-  /* Special counter for muxer to skip the first N vorbis/theora headers -
-   * they are put into codec private data, not muxed into the stream */
-  guint         xiph_headers_to_skip;
+  gboolean       send_dvd_event;
 
   /* Used for postprocessing a frame before it is pushed from the demuxer */
   GstFlowReturn (*postprocess_frame) (GstElement *element,
                                       GstMatroskaTrackContext *context,
-				      GstBuffer **buffer);
+                                      GstBuffer **buffer);
 
-  /* List of tags for this stream */
-  GstTagList   *tags;
-  /* Tags changed and should be pushed again */
-  gboolean      tags_changed;
+  //
+  // Used only by muxer
+  //
 
-  /* A GArray of GstMatroskaTrackEncoding structures which contain the
-   * encoding (compression/encryption) settings for this track, if any */
-  GArray       *encodings;
-
-  /* Whether the stream is EOS */
-  gboolean      eos;
-
-  /* any alignment we need our output buffers to have */
-  gint          alignment;
-
-  /* for compatibility with VFW files, where timestamp represents DTS */
-  gboolean      dts_only;
-
-  /* indicate that the track is raw (jpeg,raw variants) and so pts=dts */
-  gboolean		intra_only;
+  /* Special counter for muxer to skip the first N vorbis/theora headers -
+   * they are put into codec private data, not muxed into the stream */
+  guint          xiph_headers_to_skip;
 };
 
 typedef struct _GstMatroskaTrackVideoContext {
@@ -653,9 +664,6 @@ typedef struct _GstMatroskaTrackVideoContext {
 
   gboolean alpha_mode;
 
-  /* QoS */
-  GstClockTime  earliest_time;
-
   GstBuffer     *dirac_unit;
   GstVideoColorimetry colorimetry;
   GstVideoChromaSite chroma_site;
@@ -664,6 +672,10 @@ typedef struct _GstMatroskaTrackVideoContext {
   gboolean mastering_display_info_present;
 
   GstVideoContentLightLevel content_light_level;
+
+  // Only used by demuxer / parser
+  /* QoS */
+  GstClockTime  earliest_time;
 } GstMatroskaTrackVideoContext;
 
 typedef struct _GstMatroskaTrackAudioContext {
@@ -671,16 +683,17 @@ typedef struct _GstMatroskaTrackAudioContext {
 
   guint         samplerate, channels, bitdepth;
 
+  // Only used by demuxer / parser
   guint32       wvpk_block_index;
 } GstMatroskaTrackAudioContext;
 
 typedef struct _GstMatroskaTrackSubtitleContext {
   GstMatroskaTrackContext parent;
 
-  gboolean    check_utf8;     /* buffers should be valid UTF-8 */
+  // Only used by demuxer
+  gboolean    invalid_utf8;   /* work around broken files      */
   gboolean    check_markup;   /* check if buffers contain markup
                                * or plaintext and escape characters */
-  gboolean    invalid_utf8;   /* work around broken files      */
   gboolean    seen_markup_tag;  /* markup found in text */
 } GstMatroskaTrackSubtitleContext;
 
