@@ -37,11 +37,43 @@
 #define GST_IS_RTP_BIN_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_RTP_BIN))
 
+/**
+ * GstRTCPSync:
+ * @GST_RTP_BIN_RTCP_SYNC_ALWAYS: Always do inter-stream synchronization based
+ *     on RTP-Info or RTCP SR or inband NTP-64 header extension.
+ * @GST_RTP_BIN_RTCP_SYNC_INITIAL: Only do inter-stream synchronization once based
+ *     on RTP-Info or RTCP SR or inband NTP-64 header extension.
+ * @GST_RTP_BIN_RTCP_SYNC_RTP_INFO: Only do inter-stream synchronization based
+ *     on RTP-Info.
+ * @GST_RTP_BIN_RTCP_SYNC_NTP: Only do inter-stream synchronization based on
+ *     RTCP SR or inband NTP-64 header extension. (Since 1.26)
+ * @GST_RTP_BIN_RTCP_SYNC_NEVER: Never do inter-stream synchronization. (Since 1.26)
+ */
 typedef enum
 {
   GST_RTP_BIN_RTCP_SYNC_ALWAYS,
   GST_RTP_BIN_RTCP_SYNC_INITIAL,
-  GST_RTP_BIN_RTCP_SYNC_RTP
+  GST_RTP_BIN_RTCP_SYNC_RTP_INFO,
+
+  /**
+   * GstRTCPSync::ntp:
+   *
+   * Only do inter-stream synchronization based on RTCP SR or inband NTP-64
+   * header extension.
+   *
+   * Since: 1.26
+   */
+  GST_RTP_BIN_RTCP_SYNC_NTP,
+
+  /**
+   * GstRTCPSync::never:
+   *
+   * Never do inter-stream synchronization.
+   *
+   * Since: 1.26
+   */
+
+  GST_RTP_BIN_RTCP_SYNC_NEVER
 } GstRTCPSync;
 
 typedef struct _GstRtpBin GstRtpBin;
@@ -74,10 +106,14 @@ struct _GstRtpBin {
   guint32         max_dropout_time;
   guint32         max_misorder_time;
   gboolean        rfc7273_sync;
+  gboolean        add_reference_timestamp_meta;
   guint           max_streams;
   guint64         max_ts_offset_adjustment;
   gint64          max_ts_offset;
   gboolean        max_ts_offset_is_set;
+  guint64         min_ts_offset;
+  gboolean        min_ts_offset_is_set;
+  guint           ts_offset_smoothing_factor;
 
   /* a list of session */
   GSList         *sessions;
@@ -88,6 +124,16 @@ struct _GstRtpBin {
   /* the default SDES items for sessions */
   GstStructure   *sdes;
 
+  /* the default FEC decoder factories for sessions */
+  GstStructure   *fec_decoders;
+
+  /* the default FEC encoder factories for sessions */
+  GstStructure   *fec_encoders;
+
+  gboolean       update_ntp64_header_ext;
+
+  gboolean       timeout_inactive_sources;
+
   /*< private >*/
   GstRtpBinPrivate *priv;
 };
@@ -102,11 +148,16 @@ struct _GstRtpBinClass {
 
   void        (*new_jitterbuffer)     (GstRtpBin *rtpbin, GstElement *jitterbuffer, guint session, guint32 ssrc);
 
+  void        (*new_storage)     (GstRtpBin *rtpbin, GstElement *jitterbuffer, guint session);
+
   /* action signals */
   void        (*clear_pt_map)         (GstRtpBin *rtpbin);
   void        (*reset_sync)           (GstRtpBin *rtpbin);
   GstElement* (*get_session)          (GstRtpBin *rtpbin, guint session);
   RTPSession* (*get_internal_session) (GstRtpBin *rtpbin, guint session);
+  GstElement* (*get_storage)          (GstRtpBin *rtpbin, guint session);
+  GObject*    (*get_internal_storage) (GstRtpBin *rtpbin, guint session);
+  void        (*clear_ssrc)           (GstRtpBin *rtpbin, guint session, guint32 ssrc);
 
   /* session manager signals */
   void     (*on_new_ssrc)       (GstRtpBin *rtpbin, guint session, guint32 ssrc);
@@ -128,12 +179,17 @@ struct _GstRtpBinClass {
   GstElement* (*request_aux_sender)   (GstRtpBin *rtpbin, guint session);
   GstElement* (*request_aux_receiver) (GstRtpBin *rtpbin, guint session);
 
+  GstElement* (*request_fec_encoder)  (GstRtpBin *rtpbin, guint session);
+  GstElement* (*request_fec_decoder)  (GstRtpBin *rtpbin, guint session);
+
+  GstElement* (*request_jitterbuffer) (GstRtpBin *rtpbin, guint session);
+
   void     (*on_new_sender_ssrc)      (GstRtpBin *rtpbin, guint session, guint32 ssrc);
   void     (*on_sender_ssrc_active)   (GstRtpBin *rtpbin, guint session, guint32 ssrc);
-
-  guint    (*on_bundled_ssrc)         (GstRtpBin *rtpbin, guint ssrc);
 };
 
 GType gst_rtp_bin_get_type (void);
 
+GST_ELEMENT_REGISTER_DECLARE (rtpbin);
+
 #endif /* __GST_RTP_BIN_H__ */
