@@ -14,8 +14,8 @@ VCAPS="video/x-raw,width=352,height=288,framerate=15/1"
 #DEST=192.168.1.126
 DEST=localhost
 
-gst-launch-1.0 -v gstrtpbin name=rtpbin \
-           v4l2src ! videorate ! videoconvert ! $VCAPS ! ffenc_h263p ! rtph263ppay ! rtpbin.send_rtp_sink_0      \
+gst-launch-1.0 -v rtpbin name=rtpbin \
+           v4l2src ! videorate ! videoconvert ! $VCAPS ! avenc_h263p ! rtph263ppay ! rtpbin.send_rtp_sink_0      \
                      rtpbin.send_rtp_src_0 ! queue ! udpsink host=$DEST port=5000 ts-offset=$AOFFSET      \
                      rtpbin.send_rtcp_src_0 ! udpsink host=$DEST port=5001 sync=false async=false         \
                      udpsrc port=5005 ! rtpbin.recv_rtcp_sink_0                                           \
